@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2006
+ * Copyright (c) 2006, 2007
  * Nintendo Co., Ltd.
  *
  * Permission to use, copy, modify, distribute and sell this software
@@ -33,13 +33,13 @@ public:
     unsigned int release(void);
 
     // IPic
-    void startup(unsigned irq);
-    void shutdown(unsigned irq);
-    void enable(unsigned irq);
-    void disable(unsigned irq);
-    bool ack(unsigned irq);
-    void end(unsigned irq);
-    void setAffinity(unsigned irq, unsigned int cpuMask);
+    int startup(unsigned bus, unsigned irq);
+    int shutdown(unsigned bus, unsigned irq);
+    int enable(unsigned bus, unsigned irq);
+    int disable(unsigned bus, unsigned irq);
+    bool ack(int vec);
+    bool end(int vec);
+    int setAffinity(unsigned bus, unsigned irq, unsigned int cpuMask);
     unsigned int splIdle();
     unsigned int splLo();
     unsigned int splHi();
