@@ -16,7 +16,7 @@
 #include "io.h"
 #include "rtc.h"
 
-extern long hzBus;
+extern Interlocked esShutdownCount;
 
 u8 Apic::idBSP = 0;             // boot-strap processor's local APIC id
 volatile u32* Apic::localApic;  // memory-mapped address of local APIC
@@ -67,7 +67,8 @@ setImcr(u8 value)
 
 Apic::
 Apic(Mps* mps) :
-    mps(mps)
+    mps(mps),
+    hz(0)
 {
     if (!mps)
     {
@@ -106,79 +107,72 @@ Apic::
 {
 }
 
-void Apic::
-startup(unsigned irq)
+int Apic::
+startup(unsigned int bus, unsigned irq)
 {
-    setAffinity(irq, 0xff);
-    enable(irq);
+    return setAffinity(bus, irq, 0xff);
 }
 
-void Apic::
-shutdown(unsigned irq)
+int Apic::
+shutdown(unsigned int bus, unsigned irq)
 {
-    disable(irq);
+    return disable(bus, irq);
 }
 
-void Apic::
-enable(unsigned int irq)
+int Apic::
+enable(unsigned int bus, unsigned int irq)
 {
-    enable(irq, 32 + irq);
-}
-
-void Apic::
-enable(unsigned int irq, u8 vec)
-{
-    if (vec < 0x10 || 0xfe < vec)
-    {
-        return;
-    }
-
     Mps::InterruptAssignment assignment;
-    volatile u32* addr = mps->getInterruptAssignment(irq, assignment);
+    volatile u32* addr = mps->getInterruptAssignment(bus, irq, assignment);
+    u8 intin;
     if (!addr || 23 < assignment.apicINTINn)
     {
-        return;
+        return -1;
     }
-
-    addr[IOREGSEL] = IOREDTBL + 2 * assignment.apicINTINn;
-    addr[IOWIN] = (1 << 11) | (1 << 8) | vec;   // logical, lowest priority
-}
-
-void Apic::
-disable(unsigned int irq)
-{
-    disable(irq, 32 + irq);
-}
-
-void Apic::
-disable(unsigned int irq, u8 vec)
-{
-    if (vec < 0x10 || 0xfe < vec)
+    else
     {
-        return;
+        u32 mode = (1 << 11) | (1 << 8) | (32 + assignment.apicINTINn);     // logical, lowest priority
+        if (assignment.getTriggerMode() == 3)
+        {
+            mode |= (1 << 15);      // Level sensitive
+        }
+        if (assignment.getPolarity() == 3)
+        {
+            mode |= (1 << 13);      // Low active
+        }
+        addr[IOREGSEL] = IOREDTBL + 2 * assignment.apicINTINn;
+        addr[IOWIN] = mode;
     }
+    return 32 + assignment.apicINTINn;
+}
 
+int Apic::
+disable(unsigned int bus, unsigned int irq)
+{
     Mps::InterruptAssignment assignment;
-    volatile u32* addr = mps->getInterruptAssignment(irq, assignment);
+    volatile u32* addr = mps->getInterruptAssignment(bus, irq, assignment);
+    u8 intin;
     if (!addr || 23 < assignment.apicINTINn)
     {
-        return;
+        return -1;
     }
-    addr[IOREGSEL] = IOREDTBL + 2 * assignment.apicINTINn;
-
+    else
+    {
+        intin = assignment.apicINTINn;
+    }
+    addr[IOREGSEL] = IOREDTBL + 2 * intin;
     addr[IOWIN] = 1 << 16;  // mask
+    return 32 + assignment.apicINTINn;
 }
 
 bool Apic::
-ack(unsigned int irq)
+ack(int vec)
 {
-    u8 vec = 32 + irq;
-    if (vec < 0x10 || 0xfe < vec)
-    {
-        return false;
-    }
+    ASSERT(32 <= vec && vec < 32 + 24);
 
-    // XXX disable(irq, vec);
+    // Note IOAPIC ignores edge-sensitive interrupts (ISA) signaled on a
+    // masked interrupt pin (i.e. does not deliver or hold pending). So we
+    // do not mask irq here to keep the source code simple.
 
     // Note it appears qemu does not emulate ISR and we don't check ISR here.
     localApic[EOI] = 0;
@@ -186,24 +180,28 @@ ack(unsigned int irq)
     return true;
 }
 
-void Apic::
-end(unsigned int irq)
+bool Apic::
+end(int vec)
 {
-    // XXX enable(irq);
+    return true;
 }
 
-void Apic::
-setAffinity(unsigned int irq, unsigned int mask)
+int Apic::
+setAffinity(unsigned int bus, unsigned int irq, unsigned int mask)
 {
     Mps::InterruptAssignment assignment;
-    volatile u32* addr = mps->getInterruptAssignment(irq, assignment);
+    volatile u32* addr = mps->getInterruptAssignment(bus, irq, assignment);
+    u8 intin;
     if (!addr || 23 < assignment.apicINTINn)
     {
-        return;
+        return -1;
     }
-
-    addr[IOREGSEL] = IOREDTBL + 2 * assignment.apicINTINn + 1;
-    addr[IOWIN] = (mask << 24);
+    else
+    {
+        addr[IOREGSEL] = IOREDTBL + 2 * assignment.apicINTINn + 1;
+        addr[IOWIN] = (mask << 24);
+    }
+    return 32 + assignment.apicINTINn;
 }
 
 // cf. HLT opcode is 0xf4
@@ -333,46 +331,45 @@ shutdownAp(u8 id, u32 hltAP)
 unsigned Apic::
 splIdle()
 {
-    register unsigned eax;
-    __asm__ __volatile__ (
-        "pushfl\n"
-        "popl    %0\n"
-        "sti"
-        : "=a" (eax));
-    return eax;
+    unsigned x = exchange(localApic + TPR, 0 << 4);
+    __asm__ __volatile__ ("sti\n");
+    return x;
 }
 
 unsigned Apic::
 splLo()
 {
-    register unsigned eax;
-    __asm__ __volatile__ (
-        "pushfl\n"
-        "popl   %0\n"
-        "sti"
-        : "=a" (eax));
-    return eax;
+    unsigned x = exchange(localApic + TPR, 1 << 4);
+    __asm__ __volatile__ ("sti\n");
+    return x;
 }
 
 unsigned Apic::
 splHi()
 {
-    register unsigned eax;
-    __asm__ __volatile__ (
-        "pushfl\n"
-        "popl   %0\n"
-        "cli"
-        : "=a" (eax));
-    return eax;
+    unsigned x = exchange(localApic + TPR, 15 << 4);
+    __asm__ __volatile__ ("cli\n");
+    return x;
 }
 
 void Apic::
 splX(unsigned x)
 {
-    __asm__ __volatile__ (
-        "pushl   %0\n"
-        "popfl"
-        :: "r" (x));
+    switch (x)
+    {
+    case 0 << 4:
+        splIdle();
+        break;
+    case 1 << 4:
+        splLo();
+        break;
+    case 15 << 4:
+        splHi();
+        break;
+    default:
+        esPanic(__FILE__, __LINE__, "inv. spl %d\n", x);
+        break;
+    }
 }
 
 //
@@ -476,6 +473,7 @@ setTimer(int vec, long hz)
     localApic[LVT_TR] &= ~0x20000;  // one-shot
     if (0 < hz)
     {
+        this->hz = hz;
         localApic[DCR] &= ~0x0b;
         localApic[DCR] |= 0x0b;                 // divide by 1
         localApic[ICR] = busClock / hz;
@@ -483,3 +481,39 @@ setTimer(int vec, long hz)
         localApic[LVT_TR] &= ~0x10000;          // unmask
     }
 }
+
+//
+// ICallback
+//
+
+#include "8254.h"
+
+int Apic::
+invoke(int)
+{
+    if (getLocalApicID() == 0)      // is bsp?
+    {
+        Pit::tick += 10000000 / hz;
+        Alarm::invoke();
+    }
+
+    if (0 < esShutdownCount)
+    {
+        setTimer(0, 0);             // Stop local timer
+        esShutdownCount.decrement();
+        if (getLocalApicID() == 0)  // is bsp?
+        {
+            while (0 < esShutdownCount)
+            {
+                __asm__ __volatile__ ("pause\n");
+            }
+            Core::shutdown();
+        }
+        for (;;)
+        {
+            __asm__ __volatile__ ("hlt\n");
+        }
+    }
+
+    return 0;
+}
