@@ -49,6 +49,7 @@
 #define USE_SVGA
 #define GDB_STUB
 #define USE_COM1
+// #define USE_COM3
 #define USE_SB16
 // #define USE_NE2000ISA
 
@@ -74,12 +75,15 @@ namespace
     u8              loopbackBuffer[64 * 1024];
     Uart*           uart;
     Pci*            pci;
+    IStream*        stubStream;
 };
 
 const int Page::SIZE = 4096;
 const int Page::SHIFT = 12;
 const int Page::SECTOR = 512;
 
+Interlocked esShutdownCount;
+
 struct AddressRangeDesc
 {
     u64 base;   // base address
@@ -138,7 +142,7 @@ static void initAP(...)
 {
     apic->enableLocalApic();
     apic->splHi();
-    // apic->setTimer(67, 1000);
+    apic->setTimer(32, 1000);
     Core* core = new Core(sched);
     apic->started();
     core->start();
@@ -166,26 +170,6 @@ int esInit(IInterface** nameSpace)
     Cga* cga = new Cga;
     reportStream = cga;
 
-#ifdef USE_COM1
-    // COM1 for stdio
-    if (int port = ((u16*) 0x400)[0])
-    {
-        Uart* uart = new Uart(port);
-        if (uart)
-        {
-            reportStream = uart;
-        }
-    }
-#endif
-
-#ifdef GDB_STUB
-    // COM2 for gdb
-    if (int port = ((u16*) 0x400)[1])
-    {
-        uart = new Uart(port);
-    }
-#endif
-
     // Initialize 8259 anyways.
     pic = new Pic();
 
@@ -207,30 +191,76 @@ int esInit(IInterface** nameSpace)
     {
         Core::pic = pic;
     }
-    else if (mps->getProcessorCount() < 2)
-    {
-        Core::pic = pic;
-    }
     else
     {
-        // Startup APs
-        u32 hltAP = 0x30000 + *(u16*) (0x30000 + 138);
-        u32 startAP = 0x30000 + *(u16*) (0x30000 + 126);
-        *(u32*) (0x30000 + 132) = (u32) initAP;
-
-        esReport("Startap: %x\n", startAP);
-        esReport("Halt: %x\n", hltAP);
+        Core::isaBus = mps->getISABusID();
+        esReport("ISA Bus #: %d\n", Core::isaBus);
 
         apic = new Apic(mps);
         apic->busFreq();
         Core::pic = apic;
 
-        // Core::registerExceptionHandler(67, sched);
-        // apic->setTimer(67, 1000);
+        if (2 <= mps->getProcessorCount())
+        {
+            // Startup APs
+            u32 hltAP = 0x30000 + *(u16*) (0x30000 + 138);
+            u32 startAP = 0x30000 + *(u16*) (0x30000 + 126);
+            *(u32*) (0x30000 + 132) = (u32) initAP;
 
-        apic->startup(hltAP, startAP);
+            esReport("Startap: %x\n", startAP);
+            esReport("Halt: %x\n", hltAP);
+
+            apic->startupAllAP(hltAP, startAP);
+        }
+
+        Core::registerExceptionHandler(32, apic);
+        apic->setTimer(32, 1000);
     }
 
+    // Invalidate 0-2GB region for debug purposes.
+    u32* table = static_cast<u32*>(Mmu::getPointer(0x10000));
+    for (int i = 0; i < 512; ++i)
+    {
+        table[i] = 0;
+    }
+    __asm__ __volatile__ (
+        "movl   $0x10000, %%eax\n"
+        "movl   %%eax, %%cr3\n"
+        ::: "%eax");
+
+#ifdef USE_COM1
+    // COM1 for stdio
+    if (int port = ((u16*) 0x80000400)[0])
+    {
+        Uart* uart = new Uart(port, Core::isaBus, 4);
+        if (uart)
+        {
+            reportStream = uart;
+        }
+    }
+#endif
+
+#ifdef GDB_STUB
+    // COM2 for gdb
+    if (int port = ((u16*) 0x80000400)[1])
+    {
+        uart = new Uart(port);
+    }
+#endif
+
+#ifdef USE_COM3
+    // COM3 for network test stub
+    if (int port = ((u16*) 0x80000400)[2])
+    {
+        Uart* uart = new Uart(port, Core::isaBus, 4);
+        if (uart)
+        {
+            stubStream = uart;
+            ASSERT(stubStream);
+        }
+    }
+#endif
+
     // Create the default thread (stack top: 0x80010000)
     Thread* thread = new Thread(0, 0, IThread::Normal,
                                 (void*) 0x80009000, 0x80010000 - 0x80009000);
@@ -241,7 +271,7 @@ int esInit(IInterface** nameSpace)
     core->current = thread;
     core->ktcb.tcb = thread->ktcb;
 
-    pit = new Pit(1000);
+    pit = new Pit(!mps->getFloatingPointerStructure() ? 1000 : 0);
 
     // Create class store
     classStore = static_cast<IClassStore*>(new ClassStore);
@@ -313,8 +343,8 @@ int esInit(IInterface** nameSpace)
     root->bind("device/beep", static_cast<IBeep*>(pit));
 
 #ifdef USE_SVGA
-    Vesa* vesa = new Vesa((u8*) 0x8000, (u8*) 0x8200,
-                          (u8*) ((*(u16*) (0x30000 + 128)) | ((*(u16*) (0x30000 + 130)) << 4)),
+    Vesa* vesa = new Vesa((u8*) 0x80008000, (u8*) 0x80008200,
+                          (u8*) ((*(u16*) (0x80030000 + 128)) | ((*(u16*) (0x80030000 + 130)) << 4)),
                           device);
 #endif
 
@@ -329,11 +359,18 @@ int esInit(IInterface** nameSpace)
     root->bind("device/floppy", static_cast<IStream*>(fdd));
 
 #ifdef USE_SB16
-    SoundBlaster16* sb16 = new SoundBlaster16(master, slave);
-    ASSERT(static_cast<IStream*>(&sb16->inputLine));
-    ASSERT(static_cast<IStream*>(&sb16->outputLine));
-    root->bind("device/soundInput", static_cast<IStream*>(&sb16->inputLine));
-    root->bind("device/soundOutput", static_cast<IStream*>(&sb16->outputLine));
+    try
+    {
+        SoundBlaster16* sb16 = new SoundBlaster16(Core::isaBus, master, slave);
+        ASSERT(static_cast<IStream*>(&sb16->inputLine));
+        ASSERT(static_cast<IStream*>(&sb16->outputLine));
+        root->bind("device/soundInput", static_cast<IStream*>(&sb16->inputLine));
+        root->bind("device/soundOutput", static_cast<IStream*>(&sb16->outputLine));
+    }
+    catch (...)
+    {
+        esReport("sb16: not detected.\n");
+    }
 #endif
 
     // Register the loopback interface
@@ -342,11 +379,16 @@ int esInit(IInterface** nameSpace)
 
 #ifdef USE_NE2000ISA
     // Register the Ethernet interface
-    Dp8390d* ne2000 = new Dp8390d(0xc100, 10);
+    Dp8390d* ne2000 = new Dp8390d(Core::isaBus, 0xc100, 10);
     device->bind("ethernet", static_cast<IStream*>(ne2000));
 #endif
 
-    pci = new Pci(device);
+    pci = new Pci(mps, device);
+
+#ifdef USE_COM3
+    ASSERT(stubStream);
+    device->bind("com3", static_cast<IStream*>(stubStream));
+#endif
 
     Process::initialize();
 
@@ -362,7 +404,7 @@ void esSleep(s64 timeout)
 {
     Thread* thread(Thread::getCurrentThread());
 
-    if (thread && 20000 <= timeout)
+    if (thread && 20000 < timeout)
     {
         thread->sleep(timeout);
         return;
@@ -460,3 +502,20 @@ int getDebugChar()
     }
     return data;
 }
+
+void _exit(int i)
+{
+    if (!mps->getFloatingPointerStructure())
+    {
+        Core::shutdown();
+    }
+    else
+    {
+        esShutdownCount.exchange(mps->getProcessorCount());
+        Core::splIdle();
+    }
+    for (;;)
+    {
+        __asm__ __volatile__ ("hlt");
+    }
+}
