@@ -90,11 +90,11 @@ entry(unsigned long start)
 void* Thread::
 tls(unsigned size, unsigned align)
 {
-    size = (size + sizeof(void*) + align - 1) & ~(align -1);
+    unsigned x = (size + sizeof(void*) + align - 1) & ~(align -1);
 
     Ureg* ureg(static_cast<Ureg*>(param));
-    ureg->esp -= size;
-    tcb = reinterpret_cast<void*>(ureg->esp + size - sizeof(void*));
+    ureg->esp -= x;
+    tcb = reinterpret_cast<void*>(ureg->esp + size);
     *(void**) tcb = tcb;
     return reinterpret_cast<void*>(ureg->esp);
 }
