@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2006
+ * Copyright (c) 2006, 2007
  * Nintendo Co., Ltd.
  *
  * Permission to use, copy, modify, distribute and sell this software
@@ -181,6 +181,8 @@ upcall(void* self, void* base, int methodNumber, va_list ap)
         // Initialize TLS.
         memmove(reinterpret_cast<void*>(record->ureg.esp),
                 server->tlsImage, server->tlsImageSize);
+        memset(reinterpret_cast<u8*>(record->ureg.esp) + server->tlsImageSize,
+               0, server->tlsSize - server->tlsImageSize);
 
         record->push(0);                                            // param
         record->push(reinterpret_cast<unsigned>(server->focus));    // start
@@ -690,6 +692,11 @@ copyOut(UpcallRecord* record)
                         esReport(" = %p", ip);
                     }
                 }
+                else if (ip == 0)
+                {
+                    // Requested interface is not returned.
+                    **reinterpret_cast<void****>(reinterpret_cast<int*>(paramv) + paramc) = 0;
+                }
                 else
                 {
                     return EFAULT;
