@@ -64,6 +64,33 @@ sendReset(InetMessenger* m)
     conduit->accept(&v, conduit->getB());
 }
 
+// Send a reset segment: <SEQ=SND.NXT><CTL=RST>
+void StreamReceiver::
+sendReset()
+{
+    int size = 14 + 60 + sizeof(TCPHdr);  // XXX Assume MAC, IPv4
+    Handle<InetMessenger> rst = new InetMessenger(&InetReceiver::output, size, size - sizeof(TCPHdr));
+
+    TCPHdr* tcphdr = static_cast<TCPHdr*>(rst->fix(sizeof(TCPHdr)));
+    tcphdr->src = htons(socket->getLocalPort());
+    tcphdr->dst = htons(socket->getRemotePort());
+    tcphdr->seq = htonl(sendMax);
+    tcphdr->ack = 0;
+    tcphdr->flag = htons(TCPHdr::RST);
+    tcphdr->win = 0;
+    tcphdr->sum = 0;
+    tcphdr->urg = 0;
+    tcphdr->setHdrSize(sizeof(TCPHdr));
+
+    rst->setLocal(Handle<Address>(socket->getLocal()));
+    rst->setRemote(Handle<Address>(socket->getRemote()));
+    rst->setLocalPort(socket->getLocalPort());
+    rst->setRemotePort(socket->getRemotePort());
+    rst->setType(IPPROTO_TCP);
+    Visitor v(rst);
+    conduit->accept(&v, conduit->getB());
+}
+
 bool StreamReceiver::
 trim(u16& flag, TCPSeq& seq, u16& urg, long& len, long& offset)
 {
@@ -396,7 +423,7 @@ ack(TCPSeq seq, TCPSeq ack, s32 win, s32 sent, long len)
         sendLen -= len;
         sent -= len;
 
-        monitor->notifyAll();
+        notify();
     }
 
     // Update RTT estimators
@@ -453,7 +480,6 @@ urg(InetMessenger* m, TCPSeq seq, u16 urg, long len)
 }
 
 // Copy data into recvRing
-// monitor->notifyAll();
 bool StreamReceiver::
 text(InetMessenger* m, u16& flag, TCPSeq seq,
      long len, long offset)
@@ -491,7 +517,7 @@ text(InetMessenger* m, u16& flag, TCPSeq seq,
             recvNext += adv;
             recvWin -= adv;
 
-            monitor->notifyAll();
+            notify();
         }
     }
     else    // Out-of-order:
@@ -527,7 +553,7 @@ bool StreamReceiver::
 option(TCPHdr* tcphdr)
 {
     u16 flag = ntohs(tcphdr->flag);
-    s32 mss = getDefaultMSS();
+    int mss = getDefaultMSS();
 
     int optlen = tcphdr->getHdrSize() - sizeof(TCPHdr);
     u8* opt = reinterpret_cast<u8*>(tcphdr) + sizeof(TCPHdr);   // XXX use m->fix
@@ -560,8 +586,8 @@ option(TCPHdr* tcphdr)
             {
                 return false;
             }
-            mss = reinterpret_cast<TCPOptMss*>(opt)->mss;
-            if (mss == 0)
+            mss = reinterpret_cast<TCPOptMss*>(opt)->getMSS();
+            if (mss <= 0)
             {
                 return false;
             }
@@ -649,12 +675,14 @@ StateListen::input(InetMessenger* m, StreamReceiver* s)
         return false;
     }
 
+    Handle<Address> local = m->getLocal();
+
     // Clone new socket
-    Handle<Address> addr;
-    Socket* socket = new Socket(s->getSocket()->getAddressFamily(), ISocket::Stream);
-    socket->setLocal(addr = m->getLocal());
+    ASSERT(s->socket);
+    Socket* socket = new Socket(s->socket->getAddressFamily(), ISocket::Stream);
+    socket->setLocal(local);
     socket->setLocalPort(m->getLocalPort());
-    socket->setRemote(addr = m->getRemote());
+    socket->setRemote(Handle<Address>(m->getRemote()));
     socket->setRemotePort(m->getRemotePort());
 
     SocketInstaller installer(socket);
@@ -664,6 +692,7 @@ StateListen::input(InetMessenger* m, StreamReceiver* s)
     ASSERT(accepted);
 
     // Initialize ISS and sequence number variables
+    accepted->mss = accepted->getDefaultMSS(local->getPathMTU());
     accepted->cWin = accepted->getInitialCongestionWindowSize();
     accepted->ssThresh = DEF_SSTHRESH;
     accepted->iss = accepted->isn(m);
@@ -764,7 +793,7 @@ StateSynSent::input(InetMessenger* m, StreamReceiver* s)
         s->sendMaxWin = s->sendWin;
 
         // ++interface->tcpStat.currEstab;
-        s->monitor->notifyAll();
+        s->notify();
 
         s->ackNow = true;   // XXX
     }
@@ -804,6 +833,8 @@ StateSynSent::input(InetMessenger* m, StreamReceiver* s)
 
     if (flag & TCPHdr::FIN)
     {
+        ++s->recvNext;
+        s->ackNow = true;
         s->setState(stateCloseWait);
     }
 
@@ -910,7 +941,7 @@ StateSynReceived::input(InetMessenger* m, StreamReceiver* s)
                 Synchronized<IMonitor*> method(listening->monitor);
 
                 listening->accepted.addLast(s);
-                listening->monitor->notifyAll();
+                listening->notify();
             }
         }
 
@@ -933,6 +964,8 @@ StateSynReceived::input(InetMessenger* m, StreamReceiver* s)
 
     if (flag & TCPHdr::FIN)
     {
+        ++s->recvNext;
+        s->ackNow = true;
         s->setState(stateCloseWait);
     }
 
@@ -997,6 +1030,8 @@ StateEstablished::input(InetMessenger* m, StreamReceiver* s)
 
     if (flag & TCPHdr::FIN)
     {
+        ++s->recvNext;
+        s->ackNow = true;
         s->setState(stateCloseWait);
     }
 
@@ -1066,6 +1101,7 @@ StateFinWait1::input(InetMessenger* m, StreamReceiver* s)
     if (flag & TCPHdr::FIN)
     {
         // Note if FIN has been acked, TCP has been switched to stateFinWait2.
+        ++s->recvNext;
         s->ackNow = true;
         if (s->getState() == &stateFinWait1)
         {
@@ -1138,6 +1174,7 @@ StateFinWait2::input(InetMessenger* m, StreamReceiver* s)
 
     if (flag & TCPHdr::FIN)
     {
+        ++s->recvNext;
         s->ackNow = true;
         s->setState(stateTimeWait);
     }
